/*
**> Binary Option Example
**
*/

new;
library pde,pgraph;
PDEset;

#include option.src;

cls;

K = 100;
L = 110+__macheps;
sigma = 0.2;
tau = 0.25;
r = 0.05;
b = 0;

xmin = 50; xmax = 150; N = 101;
tmin =  0; tmax = tau; M = 251;

proc aProc(t,x);
  retp( 0.5*(sigma^2) .* (x^2) );
endp;

proc bProc(t,x);
  retp( b .* x );
endp;

proc cProc(t,x);
  retp( r .* ones(rows(x),1) );
endp;

proc dProc(t,x);
  retp( zeros(rows(x),1) );
endp;

proc eProc(t,x,u);
  local cnd;
  cnd = x .<= L;	
  retp( u .* cnd );
endp;

proc tminBound(t,x);
  retp( (x - K) .* ( (x - K) .>= 0 ) .* ( x .<= L) );
endp;

proc xminBound(t,x);
  retp( zeros(rows(x),1) );
endp;

proc xmaxBound(t,x);
  retp( zeros(rows(x),1) );
endp;

_PDE_SaveLastIter = 0;
_PDE_PrintIters = 50;


/* Neumann "user" condition */

call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
         &tminBound,&xminBound,&xmaxBound,0,0);


t = GenerateGrid1(tmin,tmax,M);
xstar = K;


/*
**> First Grid
**
*/

proc gridProc1(t,i);
  retp(generateGrid1(xmin,xmax,N));
endp;

call saveGrid(t,&gridProc1,"xFile");
call solvePDE2("xFile","uFile",0.5);
{S1,U1} = readPDE2("xFile","uFile",tau);


/*
**> Second Grid
**
*/

proc gridProc2(t,i);
  retp(generateGrid3(xmin,xmax,N,xstar,2));
endp;

call saveGrid(t,&gridProc2,"xFile");
call solvePDE2("xFile","uFile",0.5);
{S2,U2} = readPDE2("xFile","uFile",tau);


/*
**> Third Grid
**
*/

proc gridProc3(t,i);
  if t < tau/3;
    retp(generateGrid3(xmin,xmax,N,L,200));
  elseif t < 2*tau/3;
    retp(generateGrid3(xmin,xmax,N,L,20));
  else;
    retp(generateGrid3(xmin,xmax,N,K,2));
  endif;
endp;

call saveGrid(t,&gridProc3,"xFile");
call solvePDE2("xFile","uFile",0.5);
{S3,U3} = readPDE2("xFile","uFile",tau);

S = S1~S2~S3;
C = BarrierUOC(S,K,sigma,tau,b,r,L,0);
U = U1~U2~U3;

{d1,d2} = derivePDE(S,U);

delta = (BarrierUOC(S*1.001,K,sigma,tau,b,r,L,0)-C)./(S*0.001);
delta_ = (C-BarrierUOC(S*0.999,K,sigma,tau,b,r,L,0))./(S*0.001);
Gamma_ = (delta-delta_)./(S*0.001);

sigma_ = sigma;
sigma = 1.0001*sigma;
call saveGrid(t,&gridProc1,"xFile");
call solvePDE2("xFile","uFile",0.5);
{S1,U1} = readPDE2("xFile","uFile",tau);
call saveGrid(t,&gridProc2,"xFile");
call solvePDE2("xFile","uFile",0.5);
{S2,U2} = readPDE2("xFile","uFile",tau);
call saveGrid(t,&gridProc3,"xFile");
call solvePDE2("xFile","uFile",0.5);
{S3,U3} = readPDE2("xFile","uFile",tau);

d3 = ((U1~U2~U3)-U)./(sigma-sigma_);
Vega = (BarrierUOC(S,K,sigma,tau,b,r,L,0)-C)./(sigma-sigma_);

graphset;
  begwind;
  window(2,2,0);

  _pdate = "";
  fonts("simplex simgrma");
  _pcross = 0; _pframe = 0; _pnum = 2; _pysci = 0;
  _pnumht = 0.20; _paxht = 0.20; _ptitlht = 0.20;

  _pltype = 6|1|3;
  xtics(70,140,10,2);
  _plegstr = "Uniform grid\000Non-uniform grid\000Temporal non-uniform grid";
  xlabel("\212S]0[");
  ylabel("error");

  setwind(1);
    title("Call");
    ytics(-0.3,0.1,0.1,2);
    xy(S,C-U);

  setwind(2);
    title("Delta");
    ytics(-0.02,0.02,0.01,2);
    xy(S,delta-d1);

  setwind(3);
    title("Gamma");
    ytics(-0.0025,0.005,0.0025,2);
    xy(S,gamma_-d2);

  setwind(4);
   _pysci = 1;
    title("Vega");
    ytics(-1,1,0.5,2);
    _plegctl = {2 5 3.8 1.25};
    xy(S,vega-d3);
    graphprt("-c=1 -cf=barrier.ps");
  endwind;


