/*
**> European Option Example
**
*/


new;
library pde,pgraph;
PDEset;

#include option.src;

cls;

K = 100;
sigma = 0.2;
tau = 0.25;
r = 0.05;
b = 0;

xmin = 50; xmax = 150; N = 151;
tmin =  0; tmax = tau; M = 251;

proc aProc(t,x);
  retp( 0.5*(sigma^2) .* (x^2) );
endp;

proc bProc(t,x);
  retp( b .* x );
endp;

proc cProc(t,x);
  retp( r .* ones(rows(x),1) );
endp;

proc dProc(t,x);
  retp( zeros(rows(x),1) );
endp;

proc tminBound(t,x);
  local PayOff;
  PayOff = x - K;
  PayOff = PayOff .* (PayOff .> 0);
  retp( PayOff );
endp;

proc xminBound(t,x);
  retp( zeros(rows(x),1) );
endp;

proc xmaxBound(t,x);
  local PayOff;
  PayOff = x - K;
  PayOff = PayOff .* (PayOff .> 0);
  retp( PayOff );
endp;

proc DxmaxBound(t,x);
  retp( ones(rows(x),1) );
endp;

_PDE_SaveLastIter = 1;
_PDE_PrintIters = 50;

/* Neumann "user" condition */

call PDE(&aProc,&bProc,&cProc,&dProc,0,
         &tminBound,&xminBound,0,0,&DxmaxBound);


t = GenerateGrid1(tmin,tmax,M);
x = GenerateGrid1(xmin,xmax,N);
call solvePDE(t,x,"uFile",0.5);
S1 = readPDE("uFile","x");
U1 = readPDE("uFile","t"|tau);

proc invcdf(u);
  local delta,x;

  u = substute(u,u .== 0,1e-7);
  u = substute(u,u .== 1,1-1e-3);

  delta = exp((b-r)*tau)*u;
  x = invEuropeanCallDelta(delta,K,sigma,tau,b,r);
 
  retp(x);
endp;

x = GenerateGrid2(xmin,xmax,N,&invcdf);
call solvePDE(t,x,"uFile",0.5);
S2 = readPDE("uFile","x");
U2 = readPDE("uFile","t"|tau);

x = GenerateGrid3(xmin,xmax,N,K,2);
call solvePDE(t,x,"uFile",0.5);
S3 = readPDE("uFile","x");
U3 = readPDE("uFile","t"|tau);

S = S1~S2~S3;
U = U1~U2~U3;
U0 = EuropeanCall(S,K,sigma,tau,b,r);


t = t[1:10];
graphset;
  begwind;
  window(1,3,0);

  _pframe = 0; _pnumht = 0.25; _paxht = 0.30; _ptitlht = 0.30;
  _pnum = 2;

  ytics(50,150,25,2);
  xtics(0,0.005,0.001,2);
  xlabel("t");
  ylabel("x");

  _pde_symsiz = 2;
  _pde_symclr = 12;

  setwind(1);
    title("Grid 1");
    {_psym,_pline} = plotGrid(t,S1,9,1,0);
    draw;

  setwind(2);
    title("Grid 2");
    {_psym,_pline} = plotGrid(t,S2,9,1,0);
    draw;

  setwind(3);
    title("Grid 3");
    {_psym,_pline} = plotGrid(t,S3,9,1,0);
    draw;

    graphprt("-c=1 -cf=eur1.ps");

  endwind;


graphset;
  _pdate = "";
  _pcross = 1; _pframe = 0; _pnum = 2;
  _pltype = 6|1|3;
  xtics(80,120,10,2);
  ytics(-0.001,0.0025,0.0005,2);
  _plegstr = "Grid 1\000Grid 2\000Grid 3";
  _plegctl = {2 5 6 5};
  xlabel("\212S]0[");
  ylabel("\212Absolute numerical error");
  graphprt("-c=1 -cf=eur2.ps");
  xy(S,U0-U);